----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    22:52:09 04/15/2010 
-- Design Name: 
-- Module Name:    ClockSync - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ClockSync is
    Port ( 
			  slowdetectedout : out std_logic;
			  slowaccessout : out  STD_LOGIC;
			  cpuclk : in std_logic;
           startfast : in  STD_LOGIC;
           extclkdel : in  STD_LOGIC;
           extclk : in  STD_LOGIC;
			  rdy : in std_logic;
			  reset : in std_logic;
			  isslow : in std_logic
		);
end ClockSync;

architecture Behavioral of ClockSync is

   signal slowaccess : std_logic := '0';
	signal slowdetected : std_logic := '0';
	signal slowdone : std_logic := '0';
	
begin

		slowaccessout <= slowaccess;
		slowdetectedout <= slowdetected;

		-- detect slow cpu access on rising edge of fast CPU clock (addr signals stable)
		-- isslow signals from address selection whether the address denotes a slow access 
		-- NOTE: this means that fast CPU address setup times must allow for address selection to work before rising phi2
		DoSlowDetect: process(cpuclk, reset, isslow, slowdone)
		begin
			if (reset = '0' or slowdone = '1') then
				-- reset
				slowdetected <= '0';
			elsif rising_edge(cpuclk) then
				slowdetected <= isslow;
			end if;
		end process DoSlowDetect;
		
		-- once slow access has been detected, start it when in sync with the 
		-- slow clock, i.e. at the falling edge of extclkdel
		-- the time from extclkdel falling to phi0 (extclk) rising must be enough to 
		-- satisfy the host system's address setup time
		DoSlowAccess: process(reset, extclkdel, startfast, slowdetected)
		begin
			if (reset = '0' or (startfast = '1' 
									--and inprocess = '0'
									)) then
				slowaccess <= '0';
			elsif (falling_edge(extclkdel)) then
				slowaccess <= slowdetected;
			end if;
		end process DoSlowAccess;

		-- when slow access is done, i.e. at falling edge of phi0 (extclk),
		-- set signal slowdone. Check rdy though to hold slow access when host memory
		-- is not ready yet.
		DoSlowDone: process(extclk, startfast, slowaccess)
		begin
			if (startfast = '1') then
				slowdone <= '0';
			elsif (falling_edge(extclk)) then
				-- wait for slow access to finish
				if (rdy = '1') then
					slowdone <= slowaccess;
				end if;
			end if;
		end process DoSlowDone;

end Behavioral;

