----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    23:06:54 02/03/2010 
-- Design Name: 
-- Module Name:    PET816 - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 1.0A - First revision
-- Additional Comments: ----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-- use IEEE.numeric_std.all;
-- use ieee.std_logic_unsigned.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity PET816 is
	Port( 			
			fastclkin : in STD_LOGIC;		-- 50MHz in
			
			-- PET interface

			phi0 : in STD_LOGIC;				-- PET Phi0 in
			phi1 : out STD_LOGIC;			-- PET Phi1 out
			phi1in : in std_logic;			-- external loop from phi1
			phi2 : out STD_LOGIC;			-- PET Phi2 out
			rdy : in STD_LOGIC;
			reset : in STD_LOGIC;
			rnw : out STD_LOGIC;

			-- external delay loops
			
			loop1out : out std_logic;		-- external delay loop to loop1in
			loop1in : in std_logic;			-- external delay loop from loop1out
			loop2out : out std_logic;		-- external delay loop to loop2in
			loop2in : in std_logic;			-- external delay loop from loop2out
						
			-- 65816 interface

			-- if you integrate the '573 high address byte latch into the CPLD, comment the next line
			-- and uncomment the following line
			cpuaddrin_64k : in STD_LOGIC_VECTOR (23 downto 16);
			--cpuaddrout_64k : out STD_LOGIC_VECTOR (18 downto 16);
			
			cpuaddrin_2k : in STD_LOGIC_VECTOR (15 downto 11);
			cpuaddr_ismyio : in STD_LOGIC;		-- when 0 then addr bits 4 to 11 are all 1
			cpuaddrin_16 : in STD_LOGIC_VECTOR (3 downto 0);
			cpudata : inout STD_LOGIC_VECTOR (7 downto 0);
			
			cpuvpa : in STD_LOGIC;
			cpuvda : in STD_LOGIC;
			--cpuvp : in STD_LOGIC;
			cpurnw : in STD_LOGIC;
			cpuwnr : out STD_LOGIC;			-- inverted CPU r/-w
			
			cpuclk : out STD_LOGIC;
			
			-- control
			
			slowaddrlatch : out STD_LOGIC;			-- when 0, pass CPU address bus to PET
			nslowbusclr	:out std_logic;			-- when 0, addr PET addr is set to 0
			nslowdataenout : out STD_LOGIC;			-- when 0, pass CPU data bus to PET
			latchen: out STD_LOGIC;					-- '573 latch control (1=passthrough, 0=latch)
			
			-- config
			bootromin: in STD_LOGIC;				-- when 1 then jumper is set to enable boot rom
			
			-- fast memory interface (2 RAM modules)
			
			nramcs : out STD_LOGIC_VECTOR (1 downto 0);
			
			-- bootrom select, in-system-programming
			nromcs : out STD_LOGIC;
			nromwe : out STD_LOGIC;
						
			-- diagnostic output signal
			diag : out STD_LOGIC;
			
			-- auxiliary config input
			auxconf : in std_logic
	);
			
end PET816;

architecture Behavioral of PET816 is

	component PET816_debug is
		Port( 			
			fastclkin : in STD_LOGIC;		-- 50MHz in
			
			-- PET interface

			phi0 : in STD_LOGIC;				-- PET Phi0 in
			phi1 : out STD_LOGIC;			-- PET Phi1 out
			phi1in : in std_logic;			-- external loop from phi1
			phi2 : out STD_LOGIC;			-- PET Phi2 out
			rdy : in STD_LOGIC;
			reset : in STD_LOGIC;			
			rnw : out STD_LOGIC;
			
			-- external delay loops
			
			loop1out : out std_logic;		-- external delay loop to loop1in
			loop1in : in std_logic;			-- external delay loop from loop1out
			loop2out : out std_logic;		-- external delay loop to loop2in
			loop2in : in std_logic;			-- external delay loop from loop2out
			
			-- 65816 interface

			-- if you integrate the '573 high address byte latch into the CPLD, comment the next line
			-- and uncomment the following line
			cpuaddrin_64k : in STD_LOGIC_VECTOR (23 downto 16);
			--cpuaddrout_64k : out STD_LOGIC_VECTOR (18 downto 16);
			
			cpuaddrin_2k : in STD_LOGIC_VECTOR (15 downto 11);
			cpuaddr_ismyio : in STD_LOGIC;		-- when 0 then addr bits 4 to 11 are all 1
			cpuaddrin_16 : in STD_LOGIC_VECTOR (3 downto 0);
			cpudata : inout STD_LOGIC_VECTOR (7 downto 0);
			
			cpuvpa : in STD_LOGIC;
			cpuvda : in STD_LOGIC;
			--cpuvp : in STD_LOGIC;
			cpurnw : in STD_LOGIC;
			cpuwnr : out STD_LOGIC;			-- inverted CPU r/-w
			
			cpuclk : out STD_LOGIC;
			
			-- control
			
			slowaddrlatch : out STD_LOGIC;			-- when 0, pass CPU address bus to PET
			nslowbusclr	:out std_logic;			-- when 0, addr PET addr is set to 0
			nslowdataenout : out STD_LOGIC;			-- when 0, pass CPU data bus to PET
			latchen: out STD_LOGIC;					-- '573 latch control (1=passthrough, 0=latch)
			
			-- fast memory interface (2 RAM modules)
			
			nramcs : out STD_LOGIC_VECTOR (1 downto 0);
			
			-- bootrom select, in-system-programming
			nromcs : out STD_LOGIC;
			nromwe : out STD_LOGIC;
						
			-- diagnostic output signal
			diag : out STD_LOGIC;
			
			---------------------------------------------
			-- debug extension
			
			control_reg_init : in std_logic_vector (7 downto 0);
			clock_reg_init : in std_logic_vector (7 downto 0)
		);		
	end component;

   signal control_reg_init_val : std_logic_vector (7 downto 0);
   signal clock_reg_init_val : std_logic_vector (7 downto 0);
	
begin
	
	control_reg_init_val (7) <= '0';					-- prgrom
	control_reg_init_val (6) <= bootromin;			-- bootrom
	control_reg_init_val (5) <= '1'; 				-- slow64k
	control_reg_init_val (4) <= '1';					-- slowclk
	control_reg_init_val (3) <= '0';					-- hidebogus
	control_reg_init_val (2) <= '0';					-- fastvread
	control_reg_init_val (1 downto 0) <= "00";	-- write protect
	
	clock_reg_init_val (4 downto 0) <= "01111";	-- clock latch
	clock_reg_init_val (6 downto 5) <= "00";		-- - unused - 
	clock_reg_init_val (7) <= '1';					-- fastmode
	
	MyCore: PET816_debug port map (
			fastclkin,
			phi0,
			phi1,
			phi1in,
			phi2,
			rdy,
			reset,
			rnw,
			loop1out,
			loop1in,
			loop2out,
			loop2in,
			cpuaddrin_64k,
			cpuaddrin_2k,
			cpuaddr_ismyio,
			cpuaddrin_16,
			cpudata,			
			cpuvpa,
			cpuvda,
			cpurnw,
			cpuwnr,			
			cpuclk,
			slowaddrlatch,
			nslowbusclr,
			nslowdataenout,
			latchen,
			nramcs,
			nromcs,
			nromwe,
			diag,
			
			---------------------------------------------
			-- debug extension
			
			control_reg_init_val,
			clock_reg_init_val
	);
	
end Behavioral;

