----------------------------------------------------------------------------
--
-- 2010-apr-16 A.Fachat
-- 	added VDA/VPA outputs (currently both driven the same way)
--
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;

package fake_6502 is
  component fake_core_6502
    port (clk           :in std_logic;
          reset         :in std_logic;
          irq_in        :in std_logic;
          nmi_in        :in std_logic;
          addr_pin      :out std_logic_vector (23 downto 0);
          din           :in  std_logic_vector (7 downto 0);
          dout          :out std_logic_vector (7 downto 0);
          dout_oe       :out std_logic;
          we_pin        :out std_logic;
          rd_pin        :out std_logic;
	  vda_out	:out std_logic;
	  vpa_out	:out std_logic
         );
  end component;

end fake_6502;

----------------------------------------------------------------------------
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.fake_6502.all;


entity fake_core_6502 is
  port (clk      :in std_logic;
        reset    :in std_logic;
        irq_in   :in std_logic;
        nmi_in   :in std_logic;
        addr_pin :out std_logic_vector (23 downto 0);
        din      :in  std_logic_vector (7 downto 0);
        dout     :out std_logic_vector (7 downto 0);
        dout_oe  :out std_logic;
        we_pin   :out std_logic;
        rd_pin   :out std_logic;
  	vda_out	 :out std_logic;
  	vpa_out	 :out std_logic
       );
end fake_core_6502;


architecture fake_6502_arch of fake_core_6502 is

  signal counter       	:std_logic_vector (7 downto 0);

  signal rnw		:std_logic;

begin

	Run: process(reset, clk)
	begin
		if (reset='1') then
			addr_pin <= (others=>'1');
			dout	<= (others=>'Z');
			rnw 	<= '1';
			vda_out	<= '0';
			vpa_out	<= '0';
			counter <= "00000000";

		elsif(rising_edge(clk)) then
			case counter is 

			-- two bogus read cycles for init

			when "00000000" =>
				addr_pin <= ("000000001111111111111100");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00000001" =>
				addr_pin <= ("000000001111111111111100");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';

			-- fetch $FFFC reset vector

			when "00000010" =>
				addr_pin <= ("000000001111111111111100");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '1';
			when "00000011" =>
				addr_pin <= ("000000001111111111111101");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '1';

			-- sample opcode

				-- opcode
			when "00000100" =>
				addr_pin <= ("000000001111110100010110");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '1';
				-- parameter
			when "00000101" =>
				addr_pin <= ("000000001111110100010111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '1';
				-- bogus
			when "00000110" =>
				addr_pin <= ("000000001111110100011000");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
				-- read param from ROM
			when "00000111" =>
				addr_pin <= ("000000000000000000000000");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- write stack, read stack

			when "00001000" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00001001" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= "01010101";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00001010" =>
				addr_pin <= ("000000000000000111111110");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
			when "00001011" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- one bogus cycle

			when "00001100" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00001101" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- two bogus cycles

			when "00001110" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00001111" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00010000" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- three bogus cycles

			when "00010001" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00010010" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00010011" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00010100" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- four bogus cycles

			when "00010101" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00010110" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00010111" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '0';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00011000" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "00011001" =>
				addr_pin <= ("000000000000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- four RAM/ROM/bus cycles

				-- ram0
			when "00011010" =>
				addr_pin <= ("000000010000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- ram1
			when "00011011" =>
				addr_pin <= ("000010010000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- rom read
			when "00011100" =>
				addr_pin <= ("111110010000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- rom write
			when "00011101" =>		-- 29
				addr_pin <= ("111110010000000111111111");
				dout	<= "11110000";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bus access
			when "00011110" =>		-- 30
				addr_pin <= ("111000001000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- switch control register then again some RAM/ROM/bus cycles

				-- write control
			when "00011111" =>		-- 31
				addr_pin <= ("000000001110111111111111");
				dout	<= "10000110";  -- prgrom, fastvread, wrotect $a-,
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bogus
			when "00100000" =>		-- 32	bogus
				addr_pin <= ("000000001110111111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
				-- ram0
			when "00100001" =>		-- 33
				addr_pin <= ("000000010000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- ram1
			when "00100010" =>		-- 34
				addr_pin <= ("000010010000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- rom read
			when "00100011" =>		-- 35
				addr_pin <= ("111110010000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- rom write (works as prgrom is set
			when "00100100" =>		-- 36
				addr_pin <= ("111110010000000111111111");
				dout	<= "11110000";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bus access
			when "00100101" =>		-- 37
				addr_pin <= ("111000001000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bogus
			when "00100110" =>		-- 38
				addr_pin <= ("000000001110111111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
				-- low64k read access ($a000)
			when "00100111" =>		-- 39
				addr_pin <= ("000000001010000000000000");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- low64k write access ($a000)
			when "00101000" =>		-- 40
				addr_pin <= ("000000001010000000000000");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- low64k write access ($2000)
			when "00101001" =>		-- 41
				addr_pin <= ("000000000010000000000000");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';

			-- switch speed register then again some RAM/ROM/bus cycles

				-- write speed
			when "00101010" =>		-- 42
				addr_pin <= ("000000001110111111111110");
				dout	<= "00000011";  -- clock latch to 3
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bogus
			when "00101011" =>		-- 43
				addr_pin <= ("000000001110101111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
				-- ram0
			when "00101100" =>		-- 44
				addr_pin <= ("000000010000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- ram1
			when "00101101" =>		-- 45
				addr_pin <= ("000010010000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- rom read
			when "00101110" =>		-- 46
				addr_pin <= ("111110010000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- rom write (works as prgrom is set
			when "00101111" =>		-- 47
				addr_pin <= ("111110010000000111111111");
				dout	<= "11110000";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bus access
			when "00110000" =>		-- 48
				addr_pin <= ("111000001000000111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bogus
			when "00110001" =>		-- 49
				addr_pin <= ("000000001110111111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
				-- low64k read access ($a000)
			when "00110010" =>		-- 50
				addr_pin <= ("000000001010000000000000");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- low64k write access ($a000)
			when "00110011" =>		-- 51
				addr_pin <= ("000000001010000000000000");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- low64k write access ($2000)
			when "00110100" =>		-- 52
				addr_pin <= ("000000000010000000000000");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- video write access ($8000)
			when "00110101" =>		-- 53
				addr_pin <= ("000000001000000000000000");
				dout	<= "10101010";
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- video read access ($8000)
			when "00110110" =>		-- 54
				addr_pin <= ("000000001000000000000000");
				dout	<= "10101010";
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';

				-- write speed
			when "00110111" =>		-- 55
				addr_pin <= ("000000001110111111111110");
				dout	<= "00000010";  -- clock latch to 2
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bogus
			when "00111000" =>		-- 56
				addr_pin <= ("000000001110101111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
				-- write speed

			when "00111001" =>		-- 57
				addr_pin <= ("000000001110111111111110");
				dout	<= "00000001";  -- clock latch to 1
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bogus
			when "00111010" =>		-- 58
				addr_pin <= ("000000001110101111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';

				-- write speed
			when "00111011" =>		-- 59
				addr_pin <= ("000000001110111111111110");
				dout	<= "00000000";  -- clock latch to 0
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bogus
			when "00111100" =>		-- 60
				addr_pin <= ("000000001110101111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';

				-- write speed
			when "00111101" =>		-- 61
				addr_pin <= ("000000001110111111111110");
				dout	<= "00000100";  -- clock latch to 4
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bogus
			when "00111110" =>		-- 62
				addr_pin <= ("000000001110101111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';

				-- write speed
			when "00111111" =>		-- 63
				addr_pin <= ("000000001110111111111110");
				dout	<= "00000101";  -- clock latch to 5
				rnw 	<= '0';
				vda_out	<= '1';
				vpa_out	<= '0';
				-- bogus
			when "01000000" =>		-- 64
				addr_pin <= ("000000001110101111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			when "01000001" =>		-- 65 slow, to see if bogus is hidden
				addr_pin <= ("111110001111101111111111");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '1';
				vpa_out	<= '0';


			-- others

			when others =>
				addr_pin <= ("000000001111111111111101");
				dout	<= (others=>'Z');
				rnw 	<= '1';
				vda_out	<= '0';
				vpa_out	<= '0';
			end case;
			counter <= counter + 1;
		end if;
	end process;

	we_pin <= not(rnw);
	rd_pin <= rnw;
	dout_oe <= not(rnw);

end fake_6502_arch;

----------------------------------------------------------------------------
----------------------------------------------------------------------------

