library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
 
entity pet816_rom_tb is
end entity;
	 
architecture TB of pet816_rom_tb is
	 
    component PET816_debug
       port( 
                        fastclkin : in STD_LOGIC;               -- 50MHz in

                        -- PET interface

                        phi0 : in STD_LOGIC;                    -- PET Phi0 in
                        phi1 : out STD_LOGIC;                   -- PET Phi1 out
			phi1in : in STD_LOGIC;			-- external loop from phi1
                        phi2 : out STD_LOGIC;                   -- PET Phi2 out
                        rdy : in STD_LOGIC;
                        reset : in STD_LOGIC;

                        rnw : out STD_LOGIC;

                        -- 65816 interface

                        cpuaddrin_64k : in STD_LOGIC_VECTOR (23 downto 16);
                        cpuaddrin_2k : in STD_LOGIC_VECTOR (15 downto 11);
                        cpuaddr_ismyio : in STD_LOGIC;          -- when 0 then addr bits 4 to 11 are all 1
                        cpuaddrin_16 : in STD_LOGIC_VECTOR (3 downto 0);
                        cpudata : inout STD_LOGIC_VECTOR (7 downto 0);

                        cpuvpa : in STD_LOGIC;
                        cpuvda : in STD_LOGIC;
                        cpurnw : in STD_LOGIC;
                        cpuwnr : out STD_LOGIC;

                        cpuclk : out STD_LOGIC;

                        -- control

                        slowaddrlatch : out STD_LOGIC;                  -- on rising edge pass CPU address to PET bus
			nslowbusclr : out STD_LOGIC;			-- on low, clear PET address bus
                        nslowdataenout : out STD_LOGIC;                 -- when 0, pass CPU data bus to PET
                        latchen: out STD_LOGIC;                         -- '573 latch control (1=passthrough, 0=latch)

                        -- fast memory interface (2 RAM modules)

                        nramcs : out STD_LOGIC_VECTOR (1 downto 0);

                        -- bootrom select, in-system-programming
                        nromcs : out STD_LOGIC;
                        nromwe : out STD_LOGIC;

                        diag : out STD_LOGIC;

			control_reg_init : in std_logic_vector (7 downto 0);
			clock_reg_init : in std_logic_vector (7 downto 0)
	 	);
    end component;

   component core_6502
     port (
	  clk           :in std_logic;
          reset         :in std_logic;
          irq_in        :in std_logic;
          nmi_in        :in std_logic;
          addr_pin      :out std_logic_vector (15 downto 0);
          din           :in  std_logic_vector (7 downto 0);
          dout          :out std_logic_vector (7 downto 0);
          dout_oe       :out std_logic;
          we_pin        :out std_logic;
          rd_pin        :out std_logic;
          sync          :out std_logic;
     	  vda_out	:out std_logic;
     	  vpa_out	:out std_logic
         );
  end component;

  component pet_rom_short 
    port (
	addr  :in std_logic_vector (14 downto 0);
        data  :out std_logic_vector (7 downto 0)
    );
  end component;

  component ram_dp
      generic (
      	       addr_bits          :integer;
               data_bits          :integer;
               register_out_flag  :integer := 0;
               block_type         :integer := 0
       );
       port (reset         :in  std_logic;
            wr_clk        :in  std_logic;
            wr_en     :in  std_logic;
            wr_addr       :in  std_logic_vector (addr_bits-1 downto 0);
            wr_data       :in  std_logic_vector(data_bits-1 downto 0);
            rd_clk        :in  std_logic;
            rd_addr       :in  std_logic_vector (addr_bits-1 downto 0);
            rd_data       :out std_logic_vector(data_bits-1 downto 0)
       );
    end component;

    signal clk1mhz:		std_logic;
    signal clk2mhz:		std_logic;
    signal clksel:		std_logic;	-- 0 is 1MHz, 1 is 2MHz emulated system clock

    signal reset:   		std_logic;
    signal clk:     		std_logic;
    signal petphi0:    		std_logic;
    signal petphi1:    		std_logic;
    signal petphi2:    		std_logic;
				
    signal petrdy:		std_logic := '1';
    signal petnorom:		std_logic;
    signal petrnw:		std_logic;

    signal irq:			std_logic :='1';
    signal nmi:			std_logic :='1';
       
    signal cpuaddr:		std_logic_vector(23 downto 0);
    signal nismyio:		std_logic := '1';
    signal cpudata:		std_logic_vector(7 downto 0);

    signal cpuvda:		std_logic := '1';
    signal cpuvpa:		std_logic := '1';
    signal cpuvp:		std_logic := '1';
    signal cpurnw:		std_logic := '1';
    signal cpuwnr:		std_logic;
    signal cpuclk:		std_logic;

    signal nslowbusclr:		std_logic;
    signal slowaddrlatch:	std_logic;
    signal nslowdataen:		std_logic;
    signal latchen:		std_logic;

    signal bootromin:		std_logic := '0';

    signal nramcs:		std_logic_vector(1 downto 0);
    signal nromcs:		std_logic := '0';
    signal nromwe:		std_logic := '0';

    signal diag:		std_logic;

    signal core_clk:		std_logic;
    signal core_reset:		std_logic;
    signal core_irq:		std_logic;
    signal core_nmi:		std_logic;
    signal core_dout:		std_logic_vector(7 downto 0);
    signal core_dout_oe:	std_logic;
    signal core_wr:		std_logic;
    signal core_rd:		std_logic;
    signal core_sync:		std_logic;
    signal core_data_out:	std_logic_vector(7 downto 0);
    signal core_data_in:	std_logic_vector(7 downto 0);

    signal rom_data		:std_logic_vector(7 downto 0);
    signal from_data		:std_logic_vector(7 downto 0);

    signal slow_addr		:std_logic_vector(15 downto 0);
    signal slow_data_out	:std_logic_vector(7 downto 0);
    signal slow_data_in		:std_logic_vector(7 downto 0);

    -- slow RAM
    signal ram_rd		:std_logic;
    signal ram_wr		:std_logic;
    signal ram_data_out		:std_logic_vector(7 downto 0);

    -- fast RAM
    signal fram_rd		:std_logic;
    signal fram_wr		:std_logic;
    signal fram_data_out	:std_logic_vector(7 downto 0);

    signal control_reg_init_val :std_logic_vector(7 downto 0);
    signal clock_reg_init_val   :std_logic_vector(7 downto 0);

begin

    control_reg_init_val <= "00100000";			-- control register init value
    clock_reg_init_val <= "10000110";			-- clock register init value
    cpuaddr(23 downto 16) <= "00000000";		-- CPU bank to run on
    clksel <= '1';					-- 0 is 1MHz, 1 is 2MHz system clock

    ----------------------------------------------
    -- CPU core

    cpu: core_6502 port map (
    	core_clk,
	core_reset,
	core_irq,
	core_nmi,
	cpuaddr(15 downto 0),
	core_data_in,
	core_data_out,
	core_dout_oe,
	core_wr,
	core_rd,
	core_sync,
	cpuvda,
	cpuvpa
	);

    core_clk <= not(cpuclk);
    core_reset <= not(reset);
    core_irq <= not(irq);
    core_nmi <= not(nmi);
    cpurnw <= not(core_wr and not(core_rd));

    DataOut: process(reset, core_data_out, cpudata, nslowdataen)
    begin
	if (reset='0') then
	    cpudata <= "ZZZZZZZZ";
	    slow_data_out <= "ZZZZZZZZ";
    	else 
	    cpudata <= core_data_out;

	    if (nslowdataen = '0') then
		slow_data_out <= cpudata;
	    else
		slow_data_out <= "ZZZZZZZZ";
	    end if;

    	end if;
    end process;

    AddrOut: process(reset, cpuaddr, nslowbusclr, slowaddrlatch)
    begin
	if (reset='0') then
	    slow_addr <= "ZZZZZZZZZZZZZZZZ";
    	else 
	    if (nslowbusclr = '0') then
		slow_addr <= "0000000000000000";
	    else
		if (rising_edge(slowaddrlatch)) then
		    slow_addr <= cpuaddr(15 downto 0);
		end if;
	    end if;
    	end if;
    end process;
	
    DataIn: process(reset, cpuaddr, slow_data_in, cpudata, nslowdataen, from_data, fram_data_out)
    begin
	if (reset='0') then
		core_data_in <= "ZZZZZZZZ";
	else
		if (nslowdataen = '0') then
			core_data_in <= slow_data_in;
		else
			if (cpuaddr(15 downto 12) > "1000") then
				core_data_in <= from_data;
			else
				core_data_in <= fram_data_out;
			end if;
		end if;
	end if;
    end process;

    SlowDataIn: process(slow_addr,nslowdataen, rom_data, ram_data_out)
    begin
	if (nslowdataen = '1') then
		slow_data_in <= "ZZZZZZZZ";
	else
		if (slow_addr(15 downto 12) > "1000") then
			slow_data_in <= rom_data;
		else
			slow_data_in <= ram_data_out;
		end if;
	end if;
    end process;

    ----------------------------------------------
    -- ROM from $9000-$ffff

    rom: pet_rom_short port map (
	slow_addr(14 downto 0),
	rom_data
	);

    ----------------------------------------------
    -- ROM from $XX9000-$XXffff, (pseudo) fast ROM

    fastrom: pet_rom_short port map (
	cpuaddr(14 downto 0),
	from_data
	);

    ----------------------------------------------
    -- RAM from $0000-$8fff

    ram: ram_dp 
    generic map (
    	15, 8, 0, 0
	)
    port map (
    	core_reset,
	ram_wr,
	'1',
	slow_addr(14 downto 0),
	slow_data_out,
	ram_rd,
	slow_addr(14 downto 0),
	ram_data_out
	);

    RamSel: process(slow_addr, petrnw, petphi0)
    begin
	    if (slow_addr(15 downto 12) <= "1000") then
		    ram_rd <= petrnw;
		    ram_wr <= not(petphi0) or petrnw;
	    else
		    ram_rd <= '0';
		    ram_wr <= '1';
	    end if;
    end process;


    ----------------------------------------------
    -- Fast RAM from $XX0000-$XX8fff, XX>00

    fastram: ram_dp 
    generic map (
    	15, 8, 0, 0
	)
    port map (
    	core_reset,
	fram_wr,
	'1',
	cpuaddr(14 downto 0),
	cpudata,
	fram_rd,
	cpuaddr(14 downto 0),
	fram_data_out
	);

    FRamSel: process(cpuaddr, nramcs, cpurnw)
    begin
	    if (cpuaddr(15 downto 12) <= "1000") then
		    fram_rd <= cpurnw and not(nramcs(0));
		    fram_wr <= nramcs(0) or cpurnw;
	    else
		    fram_rd <= '0';
		    fram_wr <= '1';
	    end if;
    end process;


    ----------------------------------------------
    -- CPLD for the glue

    cpld: PET816_debug port map (
    		            clk,

                            -- PET interface

                            petphi0,
                            petphi1,
                            petphi1,
                            petphi2,
                            petrdy,
                            reset,

                            petrnw,

                            -- 65816 interface

                            cpuaddr(23 downto 16),
                            cpuaddr(15 downto 11),
			    nismyio,
                            cpuaddr(3 downto 0),

                            cpudata,

                            cpuvpa,
                            cpuvda,
                            cpurnw,
                            cpuwnr,

                            cpuclk,

                            -- control

                            slowaddrlatch,
                            nslowbusclr,
                            nslowdataen,
                            latchen,

                            -- fast memory interface (2 RAM modules)

                            nramcs,

                            -- bootrom select, in-system-programming
                            nromcs,
                            nromwe,

                            diag,

			    control_reg_init_val,
			    clock_reg_init_val
    );
	 
    	process
    	begin
		clk <= '0';
		wait for 11 ns;
		clk <= '1';
		wait for 10 ns;
    	end process;

	process
	begin
		clk1mhz <= '0';
		wait for 500 ns;
		clk1mhz <= '1';
		wait for 500 ns;
	end process;

	process
	begin
		clk2mhz <= '0';
		wait for 250 ns;
		clk2mhz <= '1';
		wait for 250 ns;
	end process;

	petphi0 <= (not(clksel) and clk1mhz) or (clksel and clk2mhz);

	process
	begin
		reset <= '0';
		wait for 155 ns;
		reset <= '1';
		wait for 10000000 ns;
	end process;

end;
