library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
 
entity pet816test_ctrl is
end entity;
	 
architecture test of pet816test_ctrl is
	 
    component pet816_tb
       port(
                control_reg_init_val :in std_logic_vector(7 downto 0);
                clock_reg_init_val   :in std_logic_vector(7 downto 0);
                cpu_addr_bank        :in std_logic_vector(7 downto 0);
                rom_addr             :out std_logic_vector (14 downto 0);
                rom_data             :in std_logic_vector (7 downto 0);
                from_addr            :out std_logic_vector (14 downto 0);
                from_data            :in std_logic_vector (7 downto 0);
                clksel               :in std_logic;      -- 0 is 1MHz, 1 is 2MHz system clock
		clkcntr		     :out std_logic_vector (15 downto 0);
		inject_invalid	     :in std_logic   -- when 1, inject invalid access
        );
    end component;

  component pet_rom_short 
    port (
	addr  :in std_logic_vector (14 downto 0);
        data  :out std_logic_vector (7 downto 0)
    );
  end component;

  constant control_reg_init_val : std_logic_vector(7 downto 0) := "00000101";
  constant clock_reg_init_val 	: std_logic_vector(7 downto 0) := "10000110";
  constant cpu_addr_bank_init	: std_logic_vector(7 downto 0) := "00000000";
  constant clksel 		: std_logic := '1';

  signal cpu_addr_bank		: std_logic_vector(7 downto 0);
  signal rom_addr		: std_logic_vector(14 downto 0);
  signal rom_data		: std_logic_vector(7 downto 0);
  signal from_addr		: std_logic_vector(14 downto 0);
  signal from_data		: std_logic_vector(7 downto 0);
  signal clkcntr		: std_logic_vector(15 downto 0);
  signal inject_invalid		: std_logic := '0';

begin

    cpu_addr_bank <= cpu_addr_bank_init;	-- CPU bank to run on

    tb: pet816_tb port map (
	control_reg_init_val,
	clock_reg_init_val,
	cpu_addr_bank,
	rom_addr,
	rom_data,
	from_addr,
	from_data,
	clksel,
	clkcntr,
	inject_invalid
    );

    rom: pet_rom_short port map (
        rom_addr,
        rom_data
    );

  process (from_addr)
  begin
    case from_addr(14 downto 0) is

      when "111111100000000" =>  from_data <= X"A9"; 	-- LDA #17
      when "111111100000001" =>  from_data <= X"11"; 
      when "111111100000010" =>  from_data <= X"8D"; 	-- STA $EBBF	 (control reg)
      when "111111100000011" =>  from_data <= X"BF";
      when "111111100000100" =>  from_data <= X"EB";
  	-- some NOPs
      when "111111100010000" =>  from_data <= X"A9";	-- LDA #1
      when "111111100010001" =>  from_data <= X"01";
      when "111111100010010" =>  from_data <= X"8D"; 	-- STA $EBBF	 (control reg)
      when "111111100010011" =>  from_data <= X"BF";
      when "111111100010100" =>  from_data <= X"EB";
	
      when "111111111111100" =>  from_data <= "00000000";  -- 7FFC = 00	-- reset vector
      when "111111111111101" =>  from_data <= "11111111";  -- 7FFD = FF
      --end_of_rom
      when others =>  from_data <= "11101010";	-- EA = NOP
    end case;
  end process;

end;
