library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
 
entity pet816test_hbog is
end entity;
	 
architecture test of pet816test_hbog is
	 
    component pet816_tb
       port(
                control_reg_init_val :in std_logic_vector(7 downto 0);
                clock_reg_init_val   :in std_logic_vector(7 downto 0);
                cpu_addr_bank        :in std_logic_vector(7 downto 0);
                rom_addr             :out std_logic_vector (14 downto 0);
                rom_data             :in std_logic_vector (7 downto 0);
                from_addr            :out std_logic_vector (14 downto 0);
                from_data            :in std_logic_vector (7 downto 0);
                clksel               :in std_logic;      -- 0 is 1MHz, 1 is 2MHz system clock
		clkcntr		     :out std_logic_vector (15 downto 0);
		inject_invalid	     :in std_logic
        );
    end component;

  component pet_rom_short 
    port (
	addr  :in std_logic_vector (14 downto 0);
        data  :out std_logic_vector (7 downto 0)
    );
  end component;

  constant control_reg_init_val : std_logic_vector(7 downto 0) := "00110000";
  constant clock_reg_init_val 	: std_logic_vector(7 downto 0) := "10000110";
  constant cpu_addr_bank_init	: std_logic_vector(7 downto 0) := "00000000";
  constant clksel 		: std_logic := '0';  -- 1MHz in a 8296 PET

  signal cpu_addr_bank		: std_logic_vector(7 downto 0);
  signal rom_addr		: std_logic_vector(14 downto 0);
  signal rom_data		: std_logic_vector(7 downto 0);
  signal from_addr		: std_logic_vector(14 downto 0);
  signal from_data		: std_logic_vector(7 downto 0);
  signal clkcntr		: std_logic_vector(15 downto 0);
  signal inject_invalid		: std_logic;

begin

    cpu_addr_bank <= cpu_addr_bank_init;	-- CPU bank to run on

    tb: pet816_tb port map (
	control_reg_init_val,
	clock_reg_init_val,
	cpu_addr_bank,
	rom_addr,
	rom_data,
	from_addr,
	from_data,
	clksel,
 	clkcntr,
	inject_invalid
    );

    from_data <= rom_data;
--    from_data <= "11111110";
--    rom: pet_rom_short port map (
--        from_addr,
--        from_data
--    );

  process (clkcntr) 
  begin
    case clkcntr is
      when X"002C" => inject_invalid <= '1';
      when X"0021" => inject_invalid <= '1';
      when X"001F" => inject_invalid <= '1';
      when X"0011" => inject_invalid <= '1';
      when X"0005" => inject_invalid <= '1';
      when X"0006" => inject_invalid <= '1';
      when others => inject_invalid <= '0';
    end case;
  end process;

  process (rom_addr)
  begin
    case rom_addr(14 downto 0) is

      when "111111100000000" =>  rom_data <= X"A9"; 	-- LDA #$38 (slow64k, slowclk, hidebogus)
      when "111111100000001" =>  rom_data <= X"38"; 	
      when "111111100000010" =>  rom_data <= X"8D"; 	-- STA $EBBF	 (control reg)
      when "111111100000011" =>  rom_data <= X"BF";
      when "111111100000100" =>  rom_data <= X"EB";
  	-- some NOPs
      when "111111100001000" =>  rom_data <= X"A9";	-- LDA #5
      when "111111100001001" =>  rom_data <= X"05";
      when "111111100001010" =>  rom_data <= X"8D"; 	-- STA $EBBE	 (clock reg)
      when "111111100001011" =>  rom_data <= X"BE";
      when "111111100001100" =>  rom_data <= X"EB";
  	-- some NOPs
      when "111111100010000" =>  rom_data <= X"A9";	-- LDA #40 (slow64k, hidebogus)
      when "111111100010001" =>  rom_data <= X"28";
      when "111111100010010" =>  rom_data <= X"8D"; 	-- STA $EBBF	 (clock reg)
      when "111111100010011" =>  rom_data <= X"BF";
      when "111111100010100" =>  rom_data <= X"EB";
 
      -- infinite loop 	
      when "111111111111000" =>  rom_data <= X"4C";  -- 7FFC = 00	-- reset vector
      when "111111111111001" =>  rom_data <= X"F8";  -- 7FFC = 00	-- reset vector
      when "111111111111010" =>  rom_data <= X"FF";  -- 7FFC = 00	-- reset vector

      -- reset vector to $FF00
      when "111111111111100" =>  rom_data <= "00000000";  -- 7FFC = 00	-- reset vector
      when "111111111111101" =>  rom_data <= "11111111";  -- 7FFD = FF
      --end_of_rom
      when others =>  rom_data <= "11101010";	-- EA = NOP
    end case;
  end process;

end;
